-- Step 1

--Create Table
--NOTE: Must change "DBUSER" to valid oracle user name
CREATE TABLE "DBUSER"."SINGLE_EXAMPLE" 
   (	"OBJECTID_1" NUMBER(10,0), 
	"OBJECTID_2" NUMBER(9,0), 
	"OBJECTID" NUMBER(9,0), 
	"PRE_DIR" CHAR(2 BYTE), 
	"STR_NAME" CHAR(20 BYTE), 
	"STR_TYPE" CHAR(4 BYTE), 
	"SUF_DIR" CHAR(2 BYTE), 
	"ROUTE_NUM" NUMBER(4,0), 
	"USE_RATING" CHAR(2 BYTE), 
	"SHAPE_LENG" NUMBER(31,11), 
	"PRE_TYPE" CHAR(15 BYTE), 
	"CENSUS_ID" NUMBER(9,0), 
	"SHAPE_LE_1" NUMBER(31,11), 
	"SHAPE_LEN" NUMBER(31,11), 
	"GEOM" "MDSYS"."SDO_GEOMETRY" 
   ) ;
/

--Step 2

-- Turn off DEFINITIONS in SQL Statements (this allows to send ampersand in the HTTP request)
SET DEF OFF;

-- Step 3

--Create  INSERT, UPDATE, and DELETE Triggers on SINGLE_EXAMPLE table
-- These triggers are called for every inserted row the logger workspace 
-- ATTENTION 1: This method works NOT with Oracle XE intallations (SDO_UTIL is not available)
-- ATTENTION 2: If this method is used for inserting many rows in a short time, a lot of traffic is generated and the FME Server will be blocked on all engines for a time
--NOTE: Must change "DBUSER" to valid oracle user name
--NOTE: Must change "myFMEServer" to valid FMEServerAdderess

--INSERT TRIGGER
create or replace
TRIGGER DBUSER.TRG_SINGLE_EXAMPLE_INSERT 
AFTER INSERT ON DBUSER.SINGLE_EXAMPLE
FOR EACH ROW
  DECLARE 
  message CLOB;
   
  -- UTL_HTTP vars
  req utl_http.req;
BEGIN
    message := 'operation=INSERT&objectid='||:new.objectid||'&'||'geometry='||SDO_UTIL.TO_WKTGEOMETRY(:new.GEOM);
    
    -- Call the Logger workspace
    req := utl_http.begin_request('http://myFMEServer/fmejobsubmitter/trigger/oracle_single_logger.fmw','POST');
    utl_http.set_header(req, 'User-Agent', 'Mozilla/4.0');
    utl_http.set_header(req,'Content-Type','application/x-www-form-urlencoded');
    utl_http.set_header(req, 'content-length', length(message));
    utl_http.write_text(req, message);
    utl_http.end_request(req);

END TRG_SINGLE_EXAMPLE_INSERT;
/

--UPDATE TRIGGER

CREATE OR REPLACE TRIGGER DBUSER.TRG_SINGLE_EXAMPLE_UPDATE 
AFTER UPDATE ON DBUSER.SINGLE_EXAMPLE
FOR EACH ROW
  DECLARE 
  message CLOB;
   -- UTL_HTTP vars
  req utl_http.req;
BEGIN
 
    message := 'operation=UPDATE&objectid='||:new.objectid||'&'||'geometry='||SDO_UTIL.TO_WKTGEOMETRY(:new.GEOM);
    
    -- Call the Logger workspace
    req := utl_http.begin_request('http://myFMEServer/fmejobsubmitter/trigger/oracle_single_logger.fmw','POST');
    utl_http.set_header(req, 'User-Agent', 'Mozilla/4.0');
    utl_http.set_header(req,'Content-Type','application/x-www-form-urlencoded');
    utl_http.set_header(req, 'content-length', length(message));
    utl_http.write_text(req, message);
    utl_http.end_request(req);

END TRG_SINGLE_EXAMPLE_UPDATE;

/

--DELETE TRIGGER

CREATE OR REPLACE TRIGGER DBUSER.TRG_SINGLE_EXAMPLE_DELETE
AFTER DELETE ON DBUSER.SINGLE_EXAMPLE
FOR EACH ROW
  DECLARE 
  message CLOB;
   -- UTL_HTTP vars
  req utl_http.req;
BEGIN
     -- For every single row the following message is sent. It contains the Feature_ID and the Geometry as WKT string
    -- ATTENTION 1: This method works NOT with Oracle XE intallations (SDO_UTIL is not available)
    -- ATTENTION 2: If this method is used for deleting many rows in a short time, a lot of traffic is generated and the FME Server will be blocked on all engines for a while.
    message := 'operation=DELETE&objectid='||:old.objectid||'&'||'geometry='||SDO_UTIL.TO_WKTGEOMETRY(:old.GEOM);
    
    -- Call the Logger workspace
    req := utl_http.begin_request('http://myFMEServer/fmejobsubmitter/trigger/oracle_single_logger.fmw','POST');
    utl_http.set_header(req, 'User-Agent', 'Mozilla/4.0');
    utl_http.set_header(req,'Content-Type','application/x-www-form-urlencoded');
    utl_http.set_header(req, 'content-length', length(message));
    utl_http.write_text(req, message);
    utl_http.end_request(req);

END TRG_SINGLE_EXAMPLE_DELETE;
